#!/usr/bin/env python3
# coding: utf-8

"""
Description: utils
Create: 2021-02-18
Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
"""

from __future__ import print_function
import os
import sys
import subprocess
import shutil
import glob


def show_info(message):
    """
    打印 info 信息
    """
    print("INFO: {}".format(message))


def show_err(message):
    """
    打印 error 信息
    """
    print("ERROR: {}".format(message))


def execute_process(command, out_stream=None, err_stream=None,
            shell=True, executable=None):
    """
    :param command: 执行的命令
    :param out_stream:
    :param err_stream:
    :param shell:默认为True
    :param executable:默认使用/bin/sh
    """
    try:
        process = subprocess.Popen(command, shell=shell,
                                   stdout=out_stream,
                                   stderr=err_stream,
                                   executable=executable)
        (stdout, stderr) = process.communicate()
        ret = process.returncode % 256
    finally:
        if stdout is not None:
            ret = stdout.decode('utf-8')
        if stderr is not None:
            ret = stderr.decode('utf-8')

    return ret


def remove(path):
    """
    删除路径
    :param path: 需要删除的路径
    """
    try:
        if os.path.islink(path):
            os.unlink(path)
        elif os.path.isdir(path):
            shutil.rmtree(path)
        elif os.path.isfile(path):
            os.remove(path)
    except OSError as err:
        show_err("Failed to remove {}".format(path))
        os._exit(1)


def globs(first_path, second_path=None, third_path=None):
    """
    将两个带有通配符的变量匹配为路径列表
    return: path list
    """
    if third_path and second_path:
        return glob.glob(os.path.join(first_path, second_path, third_path))
    if second_path:
        return glob.glob(os.path.join(first_path, second_path))
    return glob.glob(first_path)


def mkdir(path):
    """
    创建目录
    :param path: 需要创建的路径
    """
    try:
        if not os.path.exists(path):
            os.makedirs(path)
    except OSError:
        show_err("Failed to make directory: {}".format(path))
        os._exit(1)


def __copy_file(source, dest):
    """
    拷贝文件到目标路径下
    :param source: 源路径
    :param dest: 目标路径
    """
    if not os.path.exists(source):
        show_err("{} does not exist".format(source))
        os._exit(1)

    if not os.path.exists(dest):
        mkdir(dest)
    file_name = source.split("/")[-1]
    dest_file = os.path.join(dest, file_name)

    try:
        if os.path.isdir(source):
            dir_list = os.listdir(source)
            for dir_name in dir_list:
                dir_path = os.path.join(source, dir_name)
                __copy_file(dir_path, dest_file)
        else:
            if os.path.exists(dest_file):
                remove(dest_file)
            shutil.copy(source, dest, follow_symlinks=False)
    except shutil.Error as err:
        show_err("Unable to copy file: %s" % err)
        os._exit(1)


def excute_cmd(cmd):
    """Python script call shell script

    :param cmd: shell command
    :return:- 命令执行返回值
                 True：成功
                 False：代表失败
            - info：命令成功执行时返回stdout标准正常信息
                    命令执行失败时返回stderr标准错误信息
    """
    code = 0
    try:
        out_bytes = subprocess.check_output(cmd, shell=True)
    except subprocess.CalledProcessError as error:
        out_bytes = error.output  # Output generated before error
        code = error.returncode  # Return code

    # result information should delete "\n"
    info = bytes.decode(out_bytes)
    if info[-1:] == '\n':
        info = info[:-1]
    if code == 0:
        return True, info
    return False, info


def copy(source, dest, file_list=None):
    """
    拷贝文件到目标路径下
    source可以是带有通配符的文件名或文件夹名的绝对路径，file_list为空；
    当拷贝一个文件夹下的多个文件或文件夹时，
    source可以是当前文件夹的绝对路径，file_list为多个文件名组成的列表，
    列表中的文件名或文件夹名可以包含通配符
    :param file_list: 文件列表
    :param source: 源路径
    :param dest: 目标路径
    """
    if not file_list:
        file_dir_list = globs(source)
        if not file_dir_list:
            show_err("{} is not exists!".format(source))
            os._exit(1)
        for file_dir in file_dir_list:
            __copy_file(file_dir, dest)
    else:
        for file in file_list:
            file_dir_list = globs(source, file)
            if not file_dir_list:
                show_err("{}/{} is not exists!".format(source, file))
                os._exit(1)
            for file_dir in file_dir_list:
                __copy_file(file_dir, dest)
