/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2019-2020. All rights reserved.
 * Description: The header file of E2E Library including all common definitions
 * Create: 2019-06-17
 */

#ifndef E2E_COMMON_H
#define E2E_COMMON_H

#include "Std_Types.h"

#ifdef ENABLE_STD_TYPES
#include "Std_Macro.h"
#endif

#ifdef ENABLE_MEMMAP
#define E2E_START_SEC_CODE
#include "E2E_MemMap.h"
#endif

/**
 * @file
 *
 * @brief The public function of the E2E,sucs as E2E_WriteData,E2E_ReceivedData
 */
/* ------------------------------------ The return type of Docheck_Counter ------------------------------------------ */
typedef enum {
    DOCHECK_WRONGSEQUENCE = 3,
    DOCHECK_REPEATED = 2,
    DOCHECK_OKSOMELOST = 1,
    DOCHECK_OK = 0
} DoCheckCounterReturnType;

/* ------------------------------ the received data type for Profile 1, 11, 4, 7, 6 --------------------------------- */
typedef struct {
    uint8 ReceivedCounter;
    uint8 ReceivedCrc;
    uint8 ReceivedDataIDNibble;
} E2EP01P11ReceivedDataType;

#ifdef ENABLE_E2E_P04
typedef struct {
    uint32 ReceivedDataID;
    uint16 ReceivedCounter;
    uint16 ReceivedLength;
    uint32 ReceivedCrc;
} E2EP04ReceivedDataType;
#endif

#ifdef ENABLE_E2E_P44
typedef struct {
    uint32 ReceivedDataID;
    uint16 ReceivedCounter;
    uint32 ReceivedLength;
    uint32 ReceivedCrc;
} E2EP44ReceivedDataType;
#endif

#ifdef ENABLE_E2E_P04M
typedef struct {
    uint32 ReceivedDataID;
    uint16 ReceivedCounter;
    uint16 ReceivedLength;
    uint32 ReceivedCrc;
    uint32 ReceivedSourceID;
    uint8 ReceivedMessageType;
    uint8 ReceivedMessageResult;
} E2EP04mReceivedDataType;

typedef struct {
    uint32 ComputedCrc;
    uint32 SourceID;
    uint16 Length;
    uint8 MessageType;
    uint8 MessageResult;
} E2EP04mCheckLoaclData;
#endif

#ifdef ENABLE_E2E_P07
typedef struct {
    uint32 ReceivedDataID;
    uint32 ReceivedCounter;
    uint32 ReceivedLength;
    uint64 ReceivedCrc;
} E2EP07ReceivedDataType;
#endif

#ifdef ENABLE_E2E_P06
typedef struct {
    uint8 ReceivedCounter;
    uint16 ReceivedLength;
    uint16 ReceivedCrc;
} E2EP06ReceivedDataType;
#endif

/**
* @brief Protects the array/buffer to be transmitted using the E2E profile 1
* @param[in] OutputData        the pointer pointing to start address of output data block
* @param[in] InputData         the pointer pointing to start address of input data block
* @param[in] DataLength        the length of input data which means the length of data should be writed into OutputData
* @param[in] IsBigEndian       TRUE --  Write data following Big Endian Order
*                              FALSE -- Write data following Little Endian Order
*/
FUNC(void, E2E_CODE) E2E_WriteData(P2VAR(uint8, AUTOMATIC, E2E_APPL_DATA) OutputData,
    P2CONST(uint64, AUTOMATIC, E2E_APPL_DATA) InputData, CONST(uint32, AUTOMATIC) DataLength,
    CONST(boolean, AUTOMATIC) IsBigEndian);

/**
* @brief Get the data from the payload
* @param[in] InputData         the pointer pointing to start address of input data block
* @param[in] DataLength        the length of input data which means the length of data should be writed into OutputData
* @param[in] IsBigEndian       TRUE --  Write data following Big Endian Order
*                              FALSE -- Write data following Little Endian Order
* @Return: The Data obtained by the function
*/
FUNC(uint64, E2E_CODE) E2E_ReceivedData(P2CONST(uint8, AUTOMATIC, E2E_APPL_DATA) InputData,
                                        CONST(uint32, AUTOMATIC) DataLength, CONST(boolean, AUTOMATIC) IsBigEndian);

/**
* @brief Get the Profile status according to the Counter counting and related setted parameters
* @param[in] StateCounter            The local Counter storing the counter of the last valid data
* @param[in] MaxDeltaCounter         The MaxDeltaCounter setted by User
* @param[in] ReceivedCounter         The received counter
* @param[in] CounterCircleNumber     Number of cycles of a counter
* @Retval: DOCHECK_REPEATED        DeltaCounter = 0
* @Retval: DOCHECK_OK              DeltaCounter = 1
* @Retval: DOCHECK_OKSOMELOST      1 < DeltaCounter <= MaxDeltaCounter
* @Retval: DOCHECK_WRONGSEQUENCE   DeltaCounter > MaxDeltaCounter
*/
FUNC(DoCheckCounterReturnType, E2E_CODE) DoCheck_Counter(CONST(uint64, AUTOMATIC) StateCounter,
    CONST(uint64, AUTOMATIC) MaxDeltaCounter, CONST(uint64, AUTOMATIC) ReceivedCounter,
    CONST(uint64, AUTOMATIC) CounterCircleNumber);

/**
* @brief Check whether there are null pointers in the two pointers
* @param[in] A           The first pointer
* @param[in] B           The sencond pointer
* @Return: TRUE -- At least one null pointer; FALSE -- No null pointer
*/
FUNC(boolean, E2E_CODE) E2ECompare_TwoPtrIsNull(P2CONST(void, AUTOMATIC, E2E_APPL_DATA) A,
                                                P2CONST(void, AUTOMATIC, E2E_APPL_DATA) B);

/**
* @brief Check whether there are null pointers in the two pointers
* @param[in] A           The first pointer
* @param[in] B           The sencond pointer
* @param[in] C           The third pointer
* @Return: TRUE -- At least one null pointer; FALSE -- No null pointer
*/
FUNC(boolean, E2E_CODE) E2ECompare_ThreePtrIsNull(P2CONST(void, AUTOMATIC, E2E_APPL_DATA) A,
    P2CONST(void, AUTOMATIC, E2E_APPL_DATA) B, P2CONST(void, AUTOMATIC, E2E_APPL_DATA) C);

/**
* @brief Check whether the length of the received data meets the configuration requirements for Profile 4, 6, 7
* @param[in] Length          The length of received data
* @param[in] MinDataLength   Minimum allowed length
* @param[in] MaxDataLength   Maximum allowed length
* @Return: TRUE -- The length of the received data is within the allowed range
*          FALSE -- The length of the received data is not within the allowed range
*/
FUNC(boolean, E2E_CODE) E2ECheck_DynamicLength(CONST(uint32, AUTOMATIC) Length,
    CONST(uint32, AUTOMATIC) MinDataLength, CONST(uint32, AUTOMATIC) MaxDataLength);

/**
* @brief Check whether the offset meets the configuration requirements for ALL the Profiles used on CP
* @param[in] Offset          The offset of a certain field, e.g. E2E Header
* @param[in] DataLength      The length of data in Byte
* @param[in] FieldLength     The length of field that the offset refers to
* @Return: TRUE -- The offset of the field is within the allowed range
*          FALSE -- The offset of the field is not within the allowed range
*/
FUNC(boolean, E2E_CODE) E2ECheck_OffsetRange(CONST(uint32, AUTOMATIC) Offset,
    CONST(uint32, AUTOMATIC) DataLength, CONST(uint32, AUTOMATIC) FieldLength);

/**
* @brief Check whether the two data records are equal.
* @param[in] EnableCRC       Whether to compare: True -- need to compare; FALSE -- no need to compare
* @param[in] ReceivedData    The received data
* @param[in] ComputedData    The computed data
* @Return: TRUE -- Do not Enable compare OR ReceivedData is equal to ComputedData
*          FALSE -- ReceivedData is not equal to ComputedData
*/
FUNC(boolean, E2E_CODE) E2ECheck_Judgement(CONST(boolean, AUTOMATIC) EnableCRC, CONST(uint64, AUTOMATIC) ReceivedData,
                                           CONST(uint64, AUTOMATIC) ComputedData);

#ifdef ENABLE_MEMMAP
#define E2E_STOP_SEC_CODE
#include "E2E_MemMap.h"
#endif

#endif
