/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2019-2020. All rights reserved.
 * Description: The header file of E2E_P02.c including all definitions in E2E_P02.c
 * Create: 2019-06-17
 */

#ifndef E2E_P02_H
#define E2E_P02_H

#include "E2E.h"

#ifdef ENABLE_STD_TYPES
#include "Std_Macro.h"
#endif

#ifdef ENABLE_MEMMAP
#define E2E_START_SEC_CODE
#include "E2E_MemMap.h"
#endif

/**
 * @file
 *
 * @brief The header file of E2E_P02.c including all definitions in E2E_P02.c
 */
/**
* @brief Initialize the protector of the Profile 2.
 *
 * @req{ SAR-VOS-E2E-Profile02-00002,
 * VOS E2E-Profile02 shall Initialize the protection state,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009  SDR-VOS-E2E-00010,
 * }
 *
* @param[inout] StatePtr              Pointer to port/data communication state
* @retval E2E_E_INPUTERR_NULL      At least one pointer parameter is a NULL pointer
* @retval E2E_E_OK                 Function completed successfully
*/
FUNC(Std_ReturnType, E2E_CODE) E2E_P02ProtectInit(P2VAR(E2E_P02ProtectStateType, AUTOMATIC, E2E_APPL_DATA) StatePtr);

/**
* @brief Protects the array/buffer to be transmitted using the E2E profile 2
 *
 * @startuml
 * start
 * :Increment state->counter;
 * :Write counter to Data;
 * :set DataID;
 * :Calculate CRC;
 * :Write CRC to Data;
 * :Return protectresult;
 * stop
 * @enduml
 *
 * @req{ SAR-VOS-E2E-Profile02-00001,
 * VOS E2E-Profile02 shall protect the data to be transmited using the E2E profile 2,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009  SDR-VOS-E2E-00010,
 * }
 *
* @param[in] StatePtr      Pointer to port/data communication state
* @param[in] ConfigPtr     Pointer to static configuration
* @param[inout] DataPtr       Pointer to Data to be transmitted
* @retval E2E_E_INPUTERR_NULL      At least one pointer parameter is a NULL pointer
* @retval E2E_E_INPUTERR_WRONG     At least one input parameter is erroneous, e.g. out of range
* @retval E2E_E_INTERR             An internal library error has occurred
* @retval E2E_E_OK                 Function completed successfully
*/
FUNC(Std_ReturnType, E2E_CODE) E2E_P02Protect(P2CONST(E2E_P02ConfigType, AUTOMATIC, E2E_APPL_DATA) ConfigPtr,
                                              P2VAR(E2E_P02ProtectStateType, AUTOMATIC, E2E_APPL_DATA) StatePtr,
                                              P2VAR(uint8, AUTOMATIC, E2E_APPL_DATA) DataPtr);

/**
* @brief Initialize the checker of the Profile 2
 *
 * @req{ SAR-VOS-E2E-Profile01-00002,
 * VOS E2E-Profile01 shall Initialize the check state,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009  SDR-VOS-E2E-00010,
 * }
 *
* @param[inout] StatePtr      Pointer to port/data communication state
* @retval E2E_E_INPUTERR_NULL      At least one pointer parameter is a NULL pointer
* @retval E2E_E_OK                 Function completed successfully
*/
FUNC(Std_ReturnType, E2E_CODE) E2E_P02CheckInit(P2VAR(E2E_P02CheckStateType, AUTOMATIC, E2E_APPL_DATA) StatePtr);

/**
* @brief Checks the Data received using the E2E profile 2.
 *
 * @startuml
 * start
 * :set state->MaxDelataCounter;
 * :Check state->NewDataAvailable;
 * :set receivecounter and DataID;
 * :Checksum OK;
 * :check E2E_P02_process_counter;
 * :Return checkResult;
 * stop
 * @enduml
 *
 * @req{ SAR-VOS-E2E-Profile02-00003,
 * VOS E2E-Profile02 shall Checks the Data received using the E2E profile 2,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009  SDR-VOS-E2E-00010,
 * }
 *
* @param[inout] StatePtr      Pointer to port/data communication state
* @param[in] ConfigPtr     Pointer to static configuration
* @param[in]  DataPtr       Pointer to Data to be transmitted
* @retval E2E_E_INPUTERR_NULL      At least one pointer parameter is a NULL pointer
* @retval E2E_E_INPUTERR_WRONG     At least one input parameter is erroneous, e.g. out of range
* @retval E2E_E_INTERR             An internal library error has occurred
* @retval E2E_E_OK                 Function completed successfully
*/
FUNC(Std_ReturnType, E2E_CODE) E2E_P02Check(P2CONST(E2E_P02ConfigType, AUTOMATIC, E2E_APPL_DATA) ConfigPtr,
    P2VAR(E2E_P02CheckStateType, AUTOMATIC, E2E_APPL_DATA) StatePtr, P2CONST(uint8, AUTOMATIC, E2E_APPL_DATA) DataPtr);

/**
* @brief The function maps the check status of Profile 2 to a generic check status,
*        which can be used by E2E state machine check function.
 *
 * @req{ SAR-VOS-E2E-Profile02-00005,
 * VOS E2E-Profile02 shall map the check status of Profile 2 to a generic check status;which can be used by
 * E2E state machine check function,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009  SDR-VOS-E2E-00010,
 * }
 *
* @param[in] CheckReturn          Return value of the E2E_P02Check function
* @param[in] Status               Status determined by E2E_P02Check function
* @param[in] profileBehavior     FALSE: check has the legacy behavior, before R4.2
*                                TRUE: check behaves like new P4/P5/P6 profiles introduced in R4.2
* @return Profile-independent status of the reception on one single Data in one cycle
*/
FUNC(E2E_PCheckStatusType, E2E_CODE) E2E_P02MapStatusToSM(VAR(Std_ReturnType, AUTOMATIC) CheckReturn,
    VAR(E2E_P02CheckStatusType, AUTOMATIC) Status, VAR(boolean, AUTOMATIC) ProfileBehavior);

#ifdef ENABLE_MEMMAP
#define E2E_STOP_SEC_CODE
#include "E2E_MemMap.h"
#endif

#endif

