/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2019-2020. All rights reserved.
 * Description: The header file of E2E_P06.c including all definitions in E2E_P06.c
 * Create: 2019-06-17
 */

#ifndef E2E_P06_H
#define E2E_P06_H

#include "E2E.h"

#ifdef ENABLE_STD_TYPES
#include "Std_Macro.h"
#endif

#ifdef ENABLE_MEMMAP
#define E2E_START_SEC_CODE
#include "E2E_MemMap.h"
#endif
/**
 * @file
 *
 * @brief The header file of E2E_P06.c including all definitions in E2E_P06.c
 */
/**
* @brief Initialize the protector of the Profile 6.
 *
 * @req{ SAR-VOS-E2E-Profile06-00002,
 * VOS E2E-Profile06 shall Initialize the protection state,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009  SDR-VOS-E2E-00010,
 * }
 *
 * @req{ AR-iAOS-E2E-PROFILE06-00002,
 * E2E-Profile06 shall Initialize the protection state,
 * D,
 * DR-iAOS-RTF-RTFE2E-00013,
 * }
 *
* @param[inout] StatePtr              Pointer to port/data communication state
* @retval E2E_E_INPUTERR_NULL      At least one pointer parameter is a NULL pointer
* @retval E2E_E_OK                 Function completed successfully
*/
FUNC(Std_ReturnType, E2E_CODE) E2E_P06ProtectInit(P2VAR(E2E_P06ProtectStateType, AUTOMATIC, E2E_APPL_DATA) StatePtr);

/**
* @brief Protects the array/buffer to be transmitted using the E2E profile 6
 *
 * 1. Verify inputs of the check function(Config,State,Data and Length),
 * 2. Compute offset,which is equal to Config->Offset/8,
 * 3. Copy 2-byte Length on bytes Data[Offset+2...Offset+3] using Little Endian order
 * 4. Copy 1-byte State->Counter on byte Data[Offset+2] in Little Endian order,
 * 5. then Compute CRC based on DataPtr,Crc_length,CRC ,IsFirstcall,
 * 6. Copy 2-byte  CRC on bytes Data[Offset+0...Offset+1] using big Endian order
 * 7. update counter(counter++);
 * 8. end
 *
 * @startuml
 * start
 *  note right:E2E_P06Protect(Config, State, Data, Length)
 *  if(Verify inputs of the protect function) then(input ok)
 *   :compute offset;
 *   :write length;
 *   :write counter;
 *   :compute CRC;
 *   :Write CRC;
 *   :increment counter;
 *  :Return E2E_E_OK;
 *   note left:E2E_E_OK
 *  else if(null input or wrong input)
 *  :Return E2E_E_INPUTERR_NULL;
 *  note :E2E_E_INPUTERR_NULL
 *  else (wrong input)
 *  :Return E2E_E_INPUTERR_WRONG;
 *  note right:E2E_E_INPUTERR_WRONG
 *  endif
 * stop
 * @enduml
 *
 * @req{ SAR-VOS-E2E-Profile06-00001,
 * VOS E2E-Profile06 shall protect the data to be transmited using the E2E profile 6,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009  SDR-VOS-E2E-00010,
 * }
 *
 * @req{ AR-iAOS-E2E-PROFILE06-00001,
 * E2E-Profile06 shall protect the data to be transmited using the E2E profile 6,
 * D,
 * DR-iAOS-RTF-RTFE2E-00016 DR-iAOS-RTF-RTFE2E-00035 DR-iAOS-RTF-RTFE2E-00028
 * DR-iAOS-RTF-RTFE2E-00037,
 * }
 *
* @param[in] StatePtr      Pointer to port/data communication state
* @param[in] ConfigPtr     Pointer to static configuration
* @param[inout] DataPtr       Pointer to Data to be transmitted
* @retval E2E_E_INPUTERR_NULL      At least one pointer parameter is a NULL pointer
* @retval E2E_E_INPUTERR_WRONG     At least one input parameter is erroneous, e.g. out of range
* @retval E2E_E_INTERR             An internal library error has occurred
* @retval E2E_E_OK                 Function completed successfully
*/
FUNC(Std_ReturnType, E2E_CODE) E2E_P06Protect(P2CONST(E2E_P06ConfigType, AUTOMATIC, E2E_APPL_DATA) ConfigPtr,
    P2VAR(E2E_P06ProtectStateType, AUTOMATIC, E2E_APPL_DATA) StatePtr, P2VAR(uint8, AUTOMATIC, E2E_APPL_DATA) DataPtr,
    VAR(uint16, AUTOMATIC) Length);

/**
* @brief Initialize the checker of the Profile 6
 *
 * @req{ SAR-VOS-E2E-Profile06-00004,
 * VOS E2E-Profile06 shall Initialize the check state,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009 SDR-VOS-E2E-00010,
 * }
 *
 * @req{ AR-iAOS-E2E-PROFILE06-00004,
 * E2E-Profile06 shall Initialize the check state,
 * D,
 * DR-iAOS-RTF-RTFE2E-00013,
 * }
 *
* @param[inout] StatePtr      Pointer to port/data communication state
* @retval E2E_E_INPUTERR_NULL      At least one pointer parameter is a NULL pointer
* @retval E2E_E_OK                 Function completed successfully
*/
FUNC(Std_ReturnType, E2E_CODE) E2E_P06CheckInit(P2VAR(E2E_P06CheckStateType, AUTOMATIC, E2E_APPL_DATA) StatePtr);

/**
* @brief Checks the Data received using the E2E profile 6.
 *
 * 1. Verify inputs of the check function(Config,State,Data and Length),
 * 2. If NewDataAvailable == TRUE,then compute offset,else Do checks,
 * 3. Compute offset,which is equal to Config->Offset/8,
 * 4. Read bytes Data[Offset+2...Offset+3] in Big Endian order to uint16 ReceivedLength,
 * 5. Read bytes Data[Offset+4] in Big Endian order to uint16 Counter,
 * 6. Read bytes Data[Offset+0...Offset+1] using big Endian order on 2-byte  ReceivedCRC
 * 7. Compute CRC based on DataPtr,Crc_length,CRC ,IsFirstcall,Crc_StartValue16,
 * 8. Do checks:Return Status based on NewDataAvailable,ReceivedCRC with ComputedCRC,
 *    Receivedlength with length,(DeltaCounter,State->Counter.ReceivedCounter,MaxDeltaCounter),
 * 9. update Counter,which is equal to ReceiveCounter,
 * 10. end
 *
 * @startuml
 * start
 *   note right:E2E_P06Check(Config, State, Data,Length)
 *  if(Verify inputs of the check function) then(input ok)
 *   if(NewDataAvailable ==TRUE) then (false)
 *   else (true)
 *   :compute offset;
 *   :Read length;
 *   :Read counter;
 *   :Read CRC;
 *   :compute CRC;
 *    endif
 *  :Do checks;
 *  :Return E2E_E_OK;
 *   note left:E2E_E_OK
 *  else if(null input or wrong input)
 *  :Return E2E_E_INPUTERR_NULL;
 *  note :E2E_E_INPUTERR_NULL
 *  else (wrong input)
 *  :Return E2E_E_INPUTERR_WRONG;
 *  note right:E2E_E_INPUTERR_WRONG
 *  endif
 * stop
 * @enduml
 *
 * @req{ SAR-VOS-E2E-Profile06-00003,
 * VOS E2E-Profile06 shall Checks the Data received using the E2E profile 6,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009  SDR-VOS-E2E-00010,
 * }
 *
 * @req{ AR-iAOS-E2E-PROFILE06-00003,
 * E2E-Profile06 shall Checks the Data received using the E2E profile 6,
 * D,
 * DR-iAOS-RTF-RTFE2E-00016 DR-iAOS-RTF-RTFE2E-00035 DR-iAOS-RTF-RTFE2E-00028
 * DR-iAOS-RTF-RTFE2E-00037
 * }
 *
* @param[inout] StatePtr      Pointer to port/data communication state
* @param[in] ConfigPtr     Pointer to static configuration
* @param[in]  DataPtr       Pointer to Data to be transmitted
* @retval E2E_E_INPUTERR_NULL      At least one pointer parameter is a NULL pointer
* @retval E2E_E_INPUTERR_WRONG     At least one input parameter is erroneous, e.g. out of range
* @retval E2E_E_INTERR             An internal library error has occurred
* @retval E2E_E_OK                 Function completed successfully
*/
FUNC(Std_ReturnType, E2E_CODE) E2E_P06Check(P2CONST(E2E_P06ConfigType, AUTOMATIC, E2E_APPL_DATA) ConfigPtr,
    P2VAR(E2E_P06CheckStateType, AUTOMATIC, E2E_APPL_DATA) StatePtr, P2CONST(uint8, AUTOMATIC, E2E_APPL_DATA) DataPtr,
    VAR(uint16, AUTOMATIC) Length);

/**
* @brief The function maps the check status of Profile 6 to a generic check status,
*        which can be used by E2E state machine check function.
 *
 * @req{ SAR-VOS-E2E-Profile06-00005,
 * VOS E2E-Profile06 shall map the check status of Profile 6 to a generic check status; which can be used by
 * E2E state machine check function,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009  SDR-VOS-E2E-00010,
 * }
 *
 * @req{ AR-iAOS-E2E-PROFILE06-00005,
 * E2E-Profile06 shall map the check status of Profile 6 to a generic check status; which can be used
 * by E2E state machine check function,
 * D,
 * DR-iAOS-RTF-RTFE2E-00016 DR-iAOS-RTF-RTFE2E-00035 DR-iAOS-RTF-RTFE2E-00028
 * DR-iAOS-RTF-RTFE2E-00037
 * }
 *
* @param[in] CheckReturn          Return value of the E2E_P05Check function
* @param[in] Status               Status determined by E2E_P05Check function
* @return Profile-independent status of the reception on one single Data in one cycle
*/
FUNC(E2E_PCheckStatusType, E2E_CODE) E2E_P06MapStatusToSM(VAR(Std_ReturnType, AUTOMATIC) CheckReturn,
                                                          VAR(E2E_P06CheckStatusType, AUTOMATIC) Status);

#ifdef ENABLE_MEMMAP
#define E2E_STOP_SEC_CODE
#include "E2E_MemMap.h"
#endif

#endif

