/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2019-2020. All rights reserved.
 * Description: The header file of E2E_P22.c including all definitions in E2E_P22.c
 * Create: 2019-06-17
 */

#ifndef E2E_P22_H
#define E2E_P22_H

#include "E2E.h"

#ifdef ENABLE_STD_TYPES
#include "Std_Macro.h"
#endif

#ifdef ENABLE_MEMMAP
#define E2E_START_SEC_CODE
#include "E2E_MemMap.h"
#endif

/**
 * @file
 *
 * @brief The header file of E2E_P22.c including all definitions in E2E_P22.c
 */
/**
* @brief Initialize the protector of the Profile 22.
 *
 * @req{ SAR-VOS-E2E-Profile22-00002,
 * VOS E2E-Profile22 shall Initialize the protection state,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005,SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009 SDR-VOS-E2E-00010,
 * }
 *
 * @req{ AR-iAOS-E2E-PROFILE22-00002,
 * E2E-Profile22 shall Initialize the protection state,
 * D,
 * DR-iAOS-RTF-RTFE2E-00013,
 * }
 *
* @param[inout] StatePtr              Pointer to port/data communication state
* @retval E2E_E_INPUTERR_NULL      At least one pointer parameter is a NULL pointer
* @retval E2E_E_OK                 Function completed successfully
*/
FUNC(Std_ReturnType, E2E_CODE) E2E_P22ProtectInit(P2VAR(E2E_P22ProtectStateType, AUTOMATIC, E2E_APPL_DATA) StatePtr);

/**
* @brief Protects the array/buffer to be transmitted using the E2E profile 22
 *
 * 1. Verify inputs of the check function(Config,State,Data and Length),
 * 2. Copy lower 4 bits of State->Counter to data array at bit position Config->Offset+8
 * 3. then Compute CRC based on DataPtr,Crc_length,CRC ,IsFirstcall,
 * 4. Copy 1-byte ComputedCRC on bytes Data[Config->Offset/8]
 * 5. update counter(counter++);
 * 6. end
 *
 * @startuml
 * start
 * note right:E2E_P22Protect(Config, State, Data, Length)
 *  if(Verify inputs of the protect function) then(input ok)
 *  :write DataIDNIbble;
 *  :write counter;
 *  :compute CRC;
 *  :Write CRC;
 *  :increment counter;
 *  :Return E2E_E_OK;
 *   note left:E2E_E_OK
 *  else if(null input or wrong input)
 *  :Return E2E_E_INPUTERR_NULL;
 *  note :E2E_E_INPUTERR_NULL
 *  else (wrong input)
 *  :Return E2E_E_INPUTERR_WRONG;
 *  note right:E2E_E_INPUTERR_WRONG
 *  endif
 * stop
 * @enduml
 *
 * @req{ SAR-VOS-E2E-Profile22-00001,
 * VOS E2E-Profile22 shall protect the data to be transmited using the E2E profile 22,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009 SDR-VOS-E2E-00010,
 * }
 *
 * @req{ AR-iAOS-E2E-PROFILE22-00001,
 * E2E-Profile22 shall protect the data to be transmiited using the E2E profile 22,
 * D,
 * DR-iAOS-RTF-RTFE2E-00016,DR-iAOS-RTF-RTFE2E-00035,DR-iAOS-RTF-RTFE2E-00028...
 * DR-iAOS-RTF-RTFE2E-00037,
 * }
 *
* @param[in] StatePtr      Pointer to port/data communication state
* @param[in] ConfigPtr     Pointer to static configuration
* @param[inout] DataPtr       Pointer to Data to be transmitted
* @retval E2E_E_INPUTERR_NULL      At least one pointer parameter is a NULL pointer
* @retval E2E_E_INPUTERR_WRONG     At least one input parameter is erroneous, e.g. out of range
* @retval E2E_E_INTERR             An internal library error has occurred
* @retval E2E_E_OK                 Function completed successfully
*/
FUNC(Std_ReturnType, E2E_CODE) E2E_P22Protect(P2CONST(E2E_P22ConfigType, AUTOMATIC, E2E_APPL_DATA) ConfigPtr,
    P2VAR(E2E_P22ProtectStateType, AUTOMATIC, E2E_APPL_DATA) StatePtr, P2VAR(uint8, AUTOMATIC, E2E_APPL_DATA) DataPtr,
    VAR(uint16, AUTOMATIC) Length);

/**
* @brief Initialize the checker of the Profile 22
 *
 * @req{ SAR-VOS-E2E-Profile22-00004,
 * VOS E2E-Profile22 shall Initialize the check state,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009 SDR-VOS-E2E-00010,
 * }
 *
 * @req{ AR-iAOS-E2E-PROFILE22-00004,
 * E2E-Profile22 shall Initialize the check state,
 * D,
 * DR-iAOS-RTF-RTFE2E-00013,
 * }
 *
* @param[inout] StatePtr      Pointer to port/data communication state
* @retval E2E_E_INPUTERR_NULL      At least one pointer parameter is a NULL pointer
* @retval E2E_E_OK                 Function completed successfully
*/
FUNC(Std_ReturnType, E2E_CODE) E2E_P22CheckInit(P2VAR(E2E_P22CheckStateType, AUTOMATIC, E2E_APPL_DATA) StatePtr);

/**
* @brief Checks the Data received using the E2E profile 22.
 *
 * 1. Verify inputs of the check function(Config,State,Data and Length),
 * 2. If NewDataAvailable == TRUE,then compute offset,else Do checks,
 * 3. Read 4 bit counter value from bit-position Config->Offset+8 in Data in Little Endian order to uint8 Counter,
 * 4. Copy byte Data[Config->Offset/8] to local variable ReceivedCRC
 * 6. Compute CRC based on DataPtr,Crc_length,CRC ,IsFirstcall,Crc_StartValue8,
 * 7. Do checks:Return Status based on NewDataAvailable,ReveivedCRC with ComputedCRC,
 *    (DeltaCounter,State->Counter.ReceivedCounter,MaxDeltaCounter)
 * 8. update Counter,which is equal to ReceiveCounter,
 * 9. end
 *
 * @startuml
 * start
 *  note right:E2E_P022Check(Config, State, Data,Length)
 *  if(Verify inputs of the check function) then(input ok)
 *   if(NewDataAvailable ==TRUE) then (false)
 *   else (true)
 *   :Read DataIDNIbble;
 *   :Read counter;
 *   :Read CRC;
 *   :compute CRC;
 *    endif
 *  :Do checks;
 *  :Return E2E_E_OK;
 *   note left:E2E_E_OK
 *  else if(null input or wrong input)
 *  :Return E2E_E_INPUTERR_NULL;
 *  note :E2E_E_INPUTERR_NULL
 *  else (wrong input)
 *  :Return E2E_E_INPUTERR_WRONG;
 *  note right:E2E_E_INPUTERR_WRONG
 *  endif
 * stop
 * @enduml
 *
 * @req{ SAR-VOS-E2E-Profile22-00003,
 * VOS E2E-Profile22 shall Checks the Data received using the E2E profile 22,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009 SDR-VOS-E2E-00010,
 * }
 *
 * @req{ AR-iAOS-E2E-PROFILE22-00003,
 * E2E-Profile22 shall Checks the Data received using the E2E profile 22,
 * D,
 * DR-iAOS-RTF-RTFE2E-00016 DR-iAOS-RTF-RTFE2E-00035 DR-iAOS-RTF-RTFE2E-00028 DR-iAOS-RTF-RTFE2E-00037,
 * }
 *
* @param[inout] StatePtr      Pointer to port/data communication state
* @param[in] ConfigPtr     Pointer to static configuration
* @param[in]  DataPtr       Pointer to Data to be transmitted
* @retval E2E_E_INPUTERR_NULL      At least one pointer parameter is a NULL pointer
* @retval E2E_E_INPUTERR_WRONG     At least one input parameter is erroneous, e.g. out of range
* @retval E2E_E_INTERR             An internal library error has occurred
* @retval E2E_E_OK                 Function completed successfully
*/
FUNC(Std_ReturnType, E2E_CODE) E2E_P22Check(P2CONST(E2E_P22ConfigType, AUTOMATIC, E2E_APPL_DATA) ConfigPtr,
    P2VAR(E2E_P22CheckStateType, AUTOMATIC, E2E_APPL_DATA) StatePtr, P2CONST(uint8, AUTOMATIC, E2E_APPL_DATA) DataPtr,
    VAR(uint16, AUTOMATIC) Length);

/**
* @brief The function maps the check status of Profile 22 to a generic check status,
*        which can be used by E2E state machine check function.
 *
 * @req{ SAR-VOS-E2E-Profile22-00005,
 * VOS E2E-Profile22 shall map the check status of Profile 22 to a generic check status; which can be used by
 * E2E state machine check function,
 * D,
 * SDR-VOS-E2E-00001 SDR-VOS-E2E-00002 SDR-VOS-E2E-00003 SDR-VOS-E2E-00004 SDR-VOS-E2E-00005 SDR-VOS-E2E-00006
 * SDR-VOS-E2E-00007 SDR-VOS-E2E-00008 SDR-VOS-E2E-00009 SDR-VOS-E2E-00010,
 * }
 *
 * @req{ AR-iAOS-E2E-PROFILE22-00005,
 * E2E-Profile22 shall map the check status of Profile 22 to a generic check status; which can be used
 * by E2E state machine check function,
 * D,
 * DR-iAOS-RTF-RTFE2E-00016 DR-iAOS-RTF-RTFE2E-00035 DR-iAOS-RTF-RTFE2E-00028 DR-iAOS-RTF-RTFE2E-00037,
 * }
 *
* @param[in] CheckReturn          Return value of the E2E_P11Check function
* @param[in] Status               Status determined by E2E_P11Check function
* @return Profile-independent status of the reception on one single Data in one cycle
*/
FUNC(E2E_PCheckStatusType, E2E_CODE) E2E_P22MapStatusToSM(VAR(Std_ReturnType, AUTOMATIC) CheckReturn,
                                                          VAR(E2E_P22CheckStatusType, AUTOMATIC) Status);

#ifdef ENABLE_MEMMAP
#define E2E_STOP_SEC_CODE
#include "E2E_MemMap.h"
#endif

#endif
