/**
 * \file
 * \copyright (c) ESCRYPT GmbH, Bochum, Germany 2019. All rights reserved, also regarding any disposal, exploitation, reproduction, editing, distribution, as well as in the event of applications for industrial property rights.
 */

/* Global QA-C suppression */
/* PRQA S 0776, 0778 EOF #Deviation ID directive_1.1_0776_0778 */
/* PRQA S 0292 EOF #Deviation ID directive_1.1_0292 */
/* PRQA S 0857 EOF #Deviation ID directive_1.1_0857 */
/* PRQA S 2053 EOF #Deviation ID rule_18.8_2053 */
/* PRQA S 3214 EOF #Suppressed because of the ASR memmap concept */
/* PRQA S 2000 EOF #Suppressed for simplicity. Not necessary having else statement is allowed. */
/* PRQA S 5002 EOF #Suppressed as header file names are defined by ASR. */

#ifndef CRYPTO_11_HSM_KEY_LOADER_H
#define CRYPTO_11_HSM_KEY_LOADER_H

#include <ecy_hsm_host_proxy.h>

#include <Crypto_11_Hsm_Cfg.h>
#include <Crypto_11_Hsm_Internal.h>
#include <ecy_hsm_csai_shecompat.h>
#include <ecy_hsm_she.h>

#define CRYPTO_11_HSM_SHE_RAM_KEY_ID            14u

#define CRYPTO_11_HSM_SHE_KEY_EL_IN_LENGTH      ((ecy_hsm_CSAI_SHE_M1_BLOCK_SIZE_BYTES + ecy_hsm_CSAI_SHE_M2_BLOCK_SIZE_BYTES) +\
                                                ecy_hsm_CSAI_SHE_M3_BLOCK_SIZE_BYTES)

#define CRYPTO_11_HSM_SHE_KEY_EL_OUT_LENGTH     (ecy_hsm_CSAI_SHE_M4_BLOCK_SIZE_BYTES + ecy_hsm_CSAI_SHE_M5_BLOCK_SIZE_BYTES)

#define CRYPTO_11_HSM_SHE_M1_OFFSET             0u
#define CRYPTO_11_HSM_SHE_M2_OFFSET             (CRYPTO_11_HSM_SHE_M1_OFFSET + ecy_hsm_CSAI_SHE_M1_BLOCK_SIZE_BYTES)
#define CRYPTO_11_HSM_SHE_M3_OFFSET             (CRYPTO_11_HSM_SHE_M2_OFFSET + ecy_hsm_CSAI_SHE_M2_BLOCK_SIZE_BYTES)
#define CRYPTO_11_HSM_SHE_M4_OFFSET             (CRYPTO_11_HSM_SHE_M3_OFFSET + ecy_hsm_CSAI_SHE_M3_BLOCK_SIZE_BYTES)
#define CRYPTO_11_HSM_SHE_M5_OFFSET             (CRYPTO_11_HSM_SHE_M4_OFFSET + ecy_hsm_CSAI_SHE_M4_BLOCK_SIZE_BYTES)

#define CRYPTO_11_HSM_SHE_BANK_0                0u
#define CRYPTO_11_HSM_SHE_BANK_1                1u
#define CRYPTO_11_HSM_SHE_BANK_2                2u
#define CRYPTO_11_HSM_SHE_BANK_3                3u
#define CRYPTO_11_HSM_SHE_BANK_4                4u


#define CRYPTO_11_HSM_SHE_KEYSLOTNR_RAM_KEY 0u
#define CRYPTO_11_HSM_SHE_KEYSLOTNR_SIZE    1u

/*
 * Use cases of ResultBuffer
 *  - M4M5 in KeyElementSet()               = 48 Bytes
 *  - KeySlotNr + M4M5 in KeyElementSet()   = 49 Bytes
 *  - M1..M5 in KeyElementGet()             = 112 Bytes
 *
 * => Choose use case with longest buffer for CRYPTO_11_HSM_RETURN_BUFFER_SIZE
 */
#define CRYPTO_11_HSM_RETURN_BUFFER_SIZE    CRYPTO_11_HSM_SHE_KEY_EL_IN_LENGTH + CRYPTO_11_HSM_SHE_KEY_EL_OUT_LENGTH

typedef struct /* PRQA S 3630 # Struct is used internally by multiple source files */
{
    uint32 size;
    uint8 buffer[CRYPTO_11_HSM_RETURN_BUFFER_SIZE];
} ResultBuffer;

#define CRYPTO_11_HSM_START_SEC_CODE
#include <Crypto_11_Hsm_MemMap.h>

/**
 * \req [$DD 11]
 * \req [$Satisfies $ARCH KEYM_28]
 *
 * \brief Initializes the key loader component.
 *
 * \internal
 * Key loader stores state and associated data of every configured key in keyRuntimeData[]. This function initializes this
 * data to its default values.
 * Furthermore, information about key elements are stored in Crypto_11_Hsm_Config_KeyElements[]. This function initializes
 * corresponding ElementDataPtr if the following conditions are met:
 * 1) we have write access to the element
 * 2) the element id is not equal to 1
 * \endinternal
 *
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyLoader_Initialize(void);

/**
 * \req [$DD 107]
 * \req [$Satisfies $ARCH CONFIG_7]
 *
 * \brief Shuts down the key loader component.
 *
 * \internal
 * This function solely closes the key management session since the driver state forces a re-init of the crypto driver which will initialize the
 * runtime data to its default values. Hence, there is no need to do it here. Keys loaded in this session are automatically released.
 * \endinternal
 *
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyLoader_DeInit(void);

/**
 * \req [$DD 12]
 * \req [$Satisfies $SWS_Crypto 00039][$Satisfies $ARCH KEYM_9][$Satisfies $ARCH KEYM_10][$Satisfies $ARCH KEYM_11][$Satisfies $ARCH KEYM_12][$Satisfies $ARCH KEYM_13][$Satisfies $ARCH KEYM_14][$Satisfies $ARCH KEYM_18]
 * \req [$Satisfies $ARCH KEYM_24][$Satisfies $ARCH KEYM_25][$Satisfies $ARCH KEYM_26][$Satisfies $ARCH KEYM_27][$Satisfies $ARCH KEYM_28][$Satisfies $ARCH KEYM_30][$Satisfies $ARCH KEYM_32][$Satisfies $ARCH KEYM_33][$Satisfies $ARCH KEYM_49][$Satisfies $ARCH KEYM_50]
 *
 * \brief Requests and processes key loading operation.
 *
 * \internal
 * Processes key loading operation of the key keyId in the session hSession with priority jobPriority. It needs to be polled
 * as long as the error code is CRYPTO_E_BUSY. This function updates the state of the key with keyId. If the key is
 * not loaded, loading operation will be started. If the loading is not possible another unused key will be released and
 * load key will be started when the release is completed.
 * \endinternal
 *
 * \note This function is not reentrant and therefore is protected against preemption.
 *
 * \param[in] hSession        HSM session handle to be used for key management operations (load, release)
 * \param[in] jobPriority     Priority of key management operation
 * \param[in] keyId           Crypto key ID
 *
 * \retval CRYPTO_E_BUSY              Key loading has not finished, poll further
 * \retval E_OK                       Key is loaded and locked (to protect against release)
 * \retval CRYPTO_E_KEY_NOT_VALID     Key has not been set to valid through Crypto_11_Hsm_KeySetValid, and cannot be used
 * \retval E_NOT_OK                   Internal error occurred, HSM has returned unexpected error
 * \retval CRYPTO_E_KEY_NOT_AVAILABLE Key store is full. There is no space to load new key and none of the keys currently
 *                                    loaded can be released (they are currently locked or they are startup keys)
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_LoadAndLockCipherKey(CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) hSession,
                                                                            CONST(ecy_hsm_Csai_JobPriorityT, AUTOMATIC) jobPriority,
                                                                            CONST(uint32, AUTOMATIC) keyId);

/**
 * \req [$DD 62]
 * \req [$Satisfies $ARCH KEYM_37][$Satisfies $ARCH KEYM_40]
 * \req [$Satisfies $SWS_Crypto 00148][$Satisfies $SWS_Crypto 91015][$Satisfies $SWS_Crypto 91006]
 *
 * \brief Loads the key to RAM and sets the key state to KEY_STATE_UPDATING
 *
 * \param [in] pHsmKey Key to be loaded
 * \return Error code indicating whether the load operation was successfully completed
 */
FUNC(Std_ReturnType, AUTOMATIC) Crypto_11_Hsm_LoadKeyAndEnterUpdateState(
    CONSTP2CONST(Crypto_11_Hsm_HsmKey, AUTOMATIC, CRYPTO_11_HSM_CONST) pHsmKey);
/**
 * \req [$DD 60]
 * \req [$Satisfies $SWS_Crypto 00039][$Satisfies $ARCH KEYM_9][$Satisfies $ARCH KEYM_10][$Satisfies $ARCH KEYM_11][$Satisfies $ARCH KEYM_12][$Satisfies $ARCH KEYM_13][$Satisfies $ARCH KEYM_14][$Satisfies $ARCH KEYM_18]
 * \req [$Satisfies $ARCH KEYM_24][$Satisfies $ARCH KEYM_25][$Satisfies $ARCH KEYM_26][$Satisfies $ARCH KEYM_27][$Satisfies $ARCH KEYM_28][$Satisfies $ARCH KEYM_30][$Satisfies $ARCH KEYM_32][$Satisfies $ARCH KEYM_33][$Satisfies $ARCH KEYM_49][$Satisfies $ARCH KEYM_50]
 *
 * \brief Requests and processes HSM key loading operation.
 *
 * \internal
 * Processes key loading operation an HsmKey in the session hSession with priority jobPriority. It needs to be polled
 * as long as the error code is CRYPTO_E_BUSY. This function updates the state of the key. If the key is
 * not loaded, loading operation will be started. If the loading is not possible another unused key will be released and
 * load key will be started when the release is completed.
 * \endinternal
 *
 * \note This function is not reentrant and therefore is protected against preemption.
 *
 * \param[in] hSession              HSM session handle to be used for key management operations (load, release)
 * \param[in] jobPriority           Priority of key management operation
 * \param[in] pHsmKey               Pointer to HSM key
 * \param[in] enterCriticalSection  If set to TRUE, critical section is entered
 *
 * \retval CRYPTO_E_BUSY              Key loading has not finished, poll further
 * \retval E_OK                       Key is loaded and locked (to protect against release)
 * \retval CRYPTO_E_KEY_NOT_VALID     Key has not been set to valid through Crypto_11_Hsm_KeySetValid, and cannot be used
 * \retval E_NOT_OK                   Internal error occurred, HSM has returned unexpected error
 * \retval CRYPTO_E_KEY_NOT_AVAILABLE Key store is full. There is no space to load new key and none of the keys currently
 *                                    loaded can be released (they are currently locked or they are startup keys)
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_LoadAndLockHsmKey(CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) hSession,
                                                                         CONST(ecy_hsm_Csai_JobPriorityT, AUTOMATIC) jobPriority,
                                                                         CONSTP2CONST(Crypto_11_Hsm_HsmKey, AUTOMATIC, CRYPTO_11_HSM_CONST) pHsmKey,
                                                                         CONST(boolean, AUTOMATIC) enterCriticalSection);

/**
 * \req [$DD 82]
 * \req [$Satisfies $ARCH JOBS_16][$Satisfies $ARCH SHE_9][$Satisfies $ARCH KEYM_10][$Satisfies $ARCH KEYM_18]
 *
 * In some situations it is necessary to terminate ongoing key loading operation. This is needed in case that the job
 * timeout has expired and the job must unconditionally terminate. We cannot continue polling forever/until the key
 * loading is done (reasoning in FMEA) but we must safely exit the situation although we might not know the actual state
 * of the key loading. Just leaving the key loader in the same state while resetting crypto driver object will cause
 * subtle or immediate errors in certain cases in the further operation of the key loader and thus, the crypto driver.
 * Therefore this function is created that leaves keys that are at the moment in uncertain state in safe states.
 *
 * \brief This function sets key identified with cryptoKeyId in a safe state, connected releasing keys (if any) and AES
 * key counter into safe states.
 *
 * \note The function is implemented expecting that the HSM automatically loads keys (if not loaded) when job is processed.
 * Different function will need to be implemented if this behavior changes.
 *
 * \param[in] cryptoKeyId Crypto key ID.
 */
FUNC(void, CRYPTO_11_HSM_CONST) Crypto_11_Hsm_SafelyTerminateKeyLoading(CONST(uint32, AUTOMATIC) cryptoKeyId);

/**
 * \req [$DD 13]
 * \req [$Satisfies $ARCH KEYM_9]
 *
 * \brief Release key lock so that the key loader knows that the key with keyId is not used.
 *
 * \param[in] keyId Crypto key ID.
 *
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ReleaseCipherKeyLock(CONST(uint32, AUTOMATIC) keyId);

/**
 * \req [$DD 14]
 * \req [$Satisfies $ARCH KEYM_9]
 *
 * \brief Get key handle for a key that was already loaded and locked
 *
 * \note The key was loaded and locked if Crypto_11_Hsm_LoadAndLockCipherKey returned E_OK but it
 * was not released by calling Crypto_11_Hsm_ReleaseCipherKeyLock
 *
 * \param[in] keyId         Crypto key ID
 */
FUNC(ecy_hsm_Csai_KeyHandleT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_GetLockedCipherKeyHandle(CONST(uint32, AUTOMATIC) keyId);

#if (CRYPTO_11_HSM_CONFIG_HAS_SHE_SERVICES == STD_ON)
/**
 * \brief Get the HSM key ID of a crypto key element Id = 1 that was already loaded and locked
 *
 * \param[in] cryptoKeyId   Crypto key ID
 * \retval                  HsmKeyId of the requested crypto key
 */
FUNC(ecy_hsm_Csai_KeyIdT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_GetLockedHsmKeyId(CONST(uint32, AUTOMATIC) cryptoKeyId);
#endif

/**
 * \req [$DD 18]
 * \req [$Satisfies $ARCH KEYM_6][$Satisfies $ARCH KEYM_8][$Satisfies $ARCH KEYM_11][$Satisfies $ARCH KEYM_12][$Satisfies $ARCH KEYM_13][$Satisfies $ARCH KEYM_14][$Satisfies $ARCH KEYM_47][$Satisfies $ARCH KEYM_51][$Satisfies $ARCH KEYM_56][$Satisfies $ARCH KEYM_57]
 * \req [$Satisfies $ARCH KEYM_29][$Satisfies $ARCH KEYM_42][$Satisfies $SWS_Crypto 00190][$Satisfies $SWS_Crypto 00191][$Satisfies $SWS_Crypto 00192][$Satisfies $SWS_Crypto 00193]
 *
 * \brief Update an element, with configurable timeout
 *
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_UpdateElement(
    CONSTP2CONST(Crypto_11_Hsm_Config_KeyElement, AUTOMATIC, CRYPTO_11_HSM_CONST) pElement,
    CONSTP2CONST(uint8, AUTOMATIC, CRYPTO_11_HSM_CONST) pKeyData,
    CONST(uint32, AUTOMATIC) keyLength,
    CONST(uint32, AUTOMATIC) offset,
    CONST(boolean, AUTOMATIC) partialUpdate,
    CONSTP2VAR(ResultBuffer, AUTOMATIC, CRYPTO_11_HSM_VAR) resultBuffer);

/**
 * \req [$DD 67]
 * \req [$Satisfies $ARCH KEYM_6][$Satisfies $ARCH KEYM_8][$Satisfies $ARCH KEYM_11][$Satisfies $ARCH KEYM_12][$Satisfies $ARCH KEYM_13][$Satisfies $ARCH KEYM_14][$Satisfies $ARCH KEYM_29][$Satisfies $ARCH KEYM_42][$Satisfies $ARCH KEYM_37][$Satisfies $ARCH KEYM_40][$Satisfies $ARCH KEYM_52][$Satisfies $ARCH KEYM_53][$Satisfies $ARCH KEYM_58][$Satisfies $ARCH SHE_11][$Satisfies $ARCH SHE_13][$Satisfies $ARCH SHE_16][$Satisfies $ARCH SHE_17][$Satisfies $ARCH SHE_31]
 * \req [$Satisfies $SWS_Crypto 00148][$Satisfies $SWS_Crypto 91004][$Satisfies $SWS_Crypto 91006][$Satisfies $SWS_Crypto 91007][$Satisfies $SWS_Crypto 91015][$Satisfies $SWS_Crypto 00190][$Satisfies $SWS_Crypto 00191][$Satisfies $SWS_Crypto 00192][$Satisfies $SWS_Crypto 00193]
 *
 * \brief Update an HSM key element with configurable timeout
 *
 * Update a true HSM key slot either by given or newly generated key data or by an HSM internal key using a key
 * handle.If the HSM key is in correct state, enter KEY_STATE_UPDATING to protect against concurrent access. Invalidate
 * all elements that refer to this HSM key, release the key RAM copy, and try to update the key until configured timeout.
 * Update the key state at the end. The key handle hKeySource is used if it is set to a value other than
 * ecy_hsm_CSAI_INVALID_KEY_HANDLE_VALUE.
 **/
FUNC(Std_ReturnType, AUTOMATIC) Crypto_11_Hsm_UpdateHsmKey(
    CONSTP2CONST(Crypto_11_Hsm_Config_KeyElement, AUTOMATIC, CRYPTO_11_HSM_CONST) pElement,
    CONSTP2CONST(uint8, AUTOMATIC, CRYPTO_11_HSM_CONST) pKeyData,
    CONST(uint32, AUTOMATIC) keyLength,
    CONST(ecy_hsm_Csai_KeyHandleT, AUTOMATIC) hKeySource,
    CONSTP2VAR(ResultBuffer, AUTOMATIC, CRYPTO_11_HSM_VAR) resultBuffer);
/**
 * \req [$DD 20]
 * \req [$Satisfies $SWS_Crypto 91014]
 * \brief Validates crypto key with cryptoKeyId, without any checks.
 * \note  Enters and exits critical section
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ValidateKey( CONST(uint32, AUTOMATIC)cryptoKeyId );

/**
 * \req [$DD 57]
 * \req [$Satisfies $SWS_Crypto 00039]
 * \brief Invalidates all crypto keys which refer to pHsmKey. Returns boolean to indicate if any connected crypto key
 *  was valid before. This is used by some callers to determine if startup keys were flashed or not.
 *
 * \retval TRUE  At least one of the crypto keys was valid
 * \retval FALSE None of the crypto keys was valid
 *
 * \note  Enters critical section depending on enterCriticalSection parameter, added to improve performance when many
 * hsm keys need to be invalidated.
 */
FUNC(boolean, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_InvalidateCryptoKeysThatUseHsmKey(
    CONSTP2CONST(Crypto_11_Hsm_HsmKey, AUTOMATIC, CRYPTO_11_HSM_CONST) pHsmKey,
    CONST(boolean, AUTOMATIC) enterCriticalSection);

/**
 * \req [$DD 21]
 * \req [$Satisfies $ARCH KEYM_6]
 *
 * \brief Opens key management session which is used for all key management operations outside of job processing.
**/
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_OpenKeyManagementSession(void);

/**
 * \req [$DD 61]
 * \req [$Satisfies $ARCH KEYM_43]
 *
 * \brief Returns the keyElementId key element corresponding to the key
**/
FUNC(P2CONST(Crypto_11_Hsm_Config_KeyElement, AUTOMATIC, CRYPTO_11_HSM_CONST), CRYPTO_11_HSM_CODE) Crypto_11_Hsm_FindKeyElement(
    CONST(uint32, AUTOMATIC) cryptoKeyId,
    CONST(uint32, AUTOMATIC) keyElementId);

/**
 * \req [$DD 25]
 * \req [$Satisfies $ARCH RNG_2]
 *
 * \brief Re-seeds the PRNG by means of ecy_hsm_Csai_RandomSeedExtend().
**/
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_RandomSeedExtend(
    CONSTP2CONST(uint8, AUTOMATIC, CRYPTO_11_HSM_CONST) pSeed,
    CONST(uint32, AUTOMATIC) length);

#if ((CRYPTO_11_HSM_CONFIG_HAS_ALGOFAM_ECCNIST == STD_ON) || (CRYPTO_11_HSM_CONFIG_HAS_ALGOFAM_ED25519 == STD_ON))
/**
 * \req [$DD 26]
 * \req [$Satisfies $ARCH KEYM_34]
 *
 * \brief Initializes DH operation on the HSM (in CycurLib) so that secret value can be calculated in the subsequent call
 *
 * \param keyExchAlg            Algorithm to be used
 * \param hKey                  Key handle of a key pair that will be used for key exchange or ecy_hsm_CSAI_NO_KEY_HANDLE.
 * In Autosar, a "private key" is actually a key pair, so we can pass a private key handle here. If ecy_hsm_CSAI_NO_KEY_HANDLE
 * is chosen, a key pair will be generated in the HSM to be used only for the key exchange. Note that this will NOT use
 * any space in keystore dynamic memory, since it is stored in static buffers, so we do not need to release the ephemeral
 * key pair.
 * \param pOwnPublicBuffer      Buffer to place own public key (to be sent to the other key exchange party)
 * \param pOwnPublicLength      Length of the public key buffer
**/
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_DHInit(
    CONST(ecy_hsm_Csai_DhAlgorithmT, AUTOMATIC) keyExchAlg,
    CONST(ecy_hsm_Csai_KeyHandleT, AUTOMATIC) hKey,
    CONSTP2VAR(uint8, AUTOMATIC, CRYPTO_11_HSM_VAR) pOwnPublicBuffer,
    CONSTP2VAR(uint32, AUTOMATIC, CRYPTO_11_HSM_VAR) pOwnPublicLength);
#endif

#if ((CRYPTO_11_HSM_CONFIG_HAS_ALGOFAM_ECCNIST == STD_ON) || (CRYPTO_11_HSM_CONFIG_HAS_ALGOFAM_ED25519 == STD_ON))
/**
 * \req [$DD 43]
 * \req [$Satisfies $ARCH KEYM_35]
 *
 * \brief Calculates the shared value by using ecy_hsm_Csai_DhComputeExt().
**/
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_DHCompute(
        CONSTP2CONST(uint8, AUTOMATIC, CRYPTO_11_HSM_CONST) partnerPublicValuePtr,
        CONST(uint32, AUTOMATIC)                            partnerPublicValueLength,
        CONSTP2VAR(uint8, AUTOMATIC, CRYPTO_11_HSM_VAR)     sharedValue,
        CONST(uint32, AUTOMATIC)                            sharedValueLen,
        CONSTP2VAR(uint32, AUTOMATIC, CRYPTO_11_HSM_VAR)    pSharedValueLen
);

/**
 * \req [$DD 44]
 * \req [$Satisfies $ARCH KEYM_35]
 *
 * \brief Injects a temporary key exchange key pair into an HSM RAM slot.
 *
 * \param keyExchAlg        ECDH key exchange algorithm
 * \param pPrivKeyData       Pointer to private key component
 * \param privKeyLen     Pointer to private key length
 * \param pPubKeyData        Pointer to private key component
 * \param pubKeyLen      Pointer to private key length
 * \param keyProperties  Key property bitmask for injection
 * \param phKey          Pointer to key handle
 *
 * \retval E_OK                         Keypair injected successfully
 * \retval E_NOT_OK                     Keypair injection failed due to unspecified error
 * \retval CRYPTO_E_KEY_NOT_AVAILABLE   Keypair injection failed, could not release a key
 *
**/
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_InjectKeyExchangeKeyPair(
        CONST(ecy_hsm_Csai_DhAlgorithmT, AUTOMATIC)                       keyExchAlg,
        CONSTP2CONST(uint8, AUTOMATIC, CRYPTO_11_HSM_VAR)                 pPrivKeyData,
        CONST(uint32, AUTOMATIC)                                          privKeyLen,
        CONSTP2CONST(uint8, AUTOMATIC, CRYPTO_11_HSM_VAR)                 pPubKeyData,
        CONST(uint32, AUTOMATIC)                                          pubKeyLen,
        CONST(uint32, AUTOMATIC)                                          keyProperties,
        CONSTP2VAR(ecy_hsm_Csai_KeyHandleT, AUTOMATIC, CRYPTO_11_HSM_VAR) phKey);
/**
 * \req [$DD 45]
 * \req [$Satisfies $ARCH KEYM_35]
 *
 * \brief Releases space in keystore after the exchange is done. It either releases private key which was loaded to act
 * as a key pair for the exchange, or injected key pair according to flags keyPairInjected and privateKeyLocked.
 * \param pPrivKeyElement      Private key element used in the key exchange
 * \param hKeyPair             Key handle of the key pair to be released
 * \param keyPairInjected      Flag telling us that hKeyPair should be released
 * \param privateKeyLocked     Flag telling us that pPrivKeyElement should be released
**/
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_CleanKeystoreAfterExchange(
        CONSTP2CONST(Crypto_11_Hsm_Config_KeyElement, AUTOMATIC, CRYPTO_11_HSM_CONST) pPrivKeyElement,
        CONST(ecy_hsm_Csai_KeyHandleT, AUTOMATIC) hKeyPair,
        CONST(boolean, AUTOMATIC) keyPairInjected,
        CONST(boolean, AUTOMATIC) privateKeyLocked);

#endif /* ((CRYPTO_11_HSM_CONFIG_HAS_ALGOFAM_ECCNIST == STD_ON) || (CRYPTO_11_HSM_CONFIG_HAS_ALGOFAM_ED25519 == STD_ON)) */


/**
 * \req [$DD 46]
 * \req [$Satisfies $ARCH KEYM_32]
 *
 * \brief Read a public key from an HSM key slot.
 * \param[ in] pHsmKey        Pointer to the HSM key to be read
 * \param[ in] pKeyData       Pointer containing the content of the public key to be read
 * \param[out] keyLength      Length of the read public key
**/
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_LoadReadReleasePublicKey(
        CONSTP2CONST(Crypto_11_Hsm_HsmKey, AUTOMATIC, CRYPTO_11_HSM_CONST) pHsmKey,
        CONSTP2VAR(uint8, AUTOMATIC, CRYPTO_11_HSM_VAR) pKeyData,
        CONST(uint32, AUTOMATIC) keyLength);

/**
 * \req [$DD 47]
 * \req [$Satisfies $ARCH KEYM_9][$Satisfies $ARCH KEYM_18]
 *
 * \brief Reset the flag indicating that the HSM RAM keystore is full
 *
 * The Crypto Driver can detect when the HSM RAM keystore is full when the first load request fails with a return
 * code indicating that the keystore is full. After that, keys can be released to make space free. This happens
 * during the invocation of the main function for asynchronous jobs. In the next invocation of the main function
 * the jobs making space free can load its key. If another ProcessJob comes in the meanwhile it could take this
 * space before the original job is able to load its key. In order to prevent this, a isKeystoreFull flag has been
 * created. This flag is reset through this function at the beginning of every main function invocation and
 * has the purpose to force every incoming job to first release a further key, if the keystore is full, instead of
 * stealing the space from another job until the release operation is completed (this means, until the invocation
 * of the next main function).
**/
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyLoader_ResetIsKeystoreFull(void);

/**
 * \req [$DD 50]
 * \req [$Satisfies $ARCH KEYM_6]
 *
 * \brief Returns the internal session for key management operations.
**/
FUNC(ecy_hsm_Csai_SessionHandleT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_GetKeyManagementSession(void);

/**
 * \req [$DD 63]
 * \req [$Satisfies $ARCH KEYM_37][$Satisfies $ARCH KEYM_40]
 * \req [$Satisfies $SWS_Crypto 00203][$Satisfies $SWS_Crypto 00204][$Satisfies $SWS_Crypto 00148][$Satisfies $SWS_Crypto 91006][$Satisfies $SWS_Crypto 91015]
 *
 * \brief Returns pointers to a key element's host RAM buffer and size
 *
 * \param[in]  pElement  Key element to be read from host RAM buffer
 * \param[out] ppData    Pointer to the buffer containing the key element
 * \param[out] pSize     Pointer to the size of the key element
 *
 * \retval E_OK                              Read operation was successful
 * \retval CRYPTO_E_KEY_NOT_AVAILABLE        Key element has never been initialized before
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_GetPtrToHostData(
    CONSTP2CONST(Crypto_11_Hsm_Config_KeyElement, AUTOMATIC, CRYPTO_11_HSM_CONST) pElement,
    CONSTP2VAR(P2CONST(uint8, AUTOMATIC, CRYPTO_11_HSM_CONST), AUTOMATIC, AUTOMATIC) ppData,
    CONSTP2VAR(uint32, AUTOMATIC, CRYPTO_11_HSM_VAR) pSize);

/**
 * \req [$DD 64]
 * \req [$Satisfies $ARCH KEYM_37][$Satisfies $ARCH KEYM_40]
 * \req [$Satisfies $SWS_Crypto 00203][$Satisfies $SWS_Crypto 00204][$Satisfies $SWS_Crypto 00148][$Satisfies $SWS_Crypto 91006][$Satisfies $SWS_Crypto 91015]
 *
 * \brief Returns pointers to a key element's host RAM buffer and size for persistent key elements in secure data storage
 *
 * If the key elements are not loaded yet, they are loaded from HSM secure data storage. Otherwise only the pointer
 * to the already loaded/updated key element is given
 *
 * \param[in]  pElement  Key element to be read (from secure data storage or host core mirror)
 * \param[out] ppData    Pointer to the buffer containing the key element
 * \param[out] pSize     Pointer to the size of the key element
 *
 * \retval E_OK                          Read operation was successful
 * \retval CRYPTO_E_KEY_NOT_AVAILABLE    Key element has never been initialized before or cannot be retrieved from HSM
 *                                       secure data store
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_LoadAndGetPtrToHostData(
    CONSTP2CONST(Crypto_11_Hsm_Config_KeyElement, AUTOMATIC, CRYPTO_11_HSM_CONST) pElement,
    CONSTP2VAR(P2CONST(uint8, AUTOMATIC, CRYPTO_11_HSM_CONST), AUTOMATIC, AUTOMATIC) ppData,
    CONSTP2VAR(uint32, AUTOMATIC, CRYPTO_11_HSM_VAR) pSize);

/**
 * \req [$DD 65]
 * \req [$Satisfies $ARCH KEYM_37][$Satisfies $ARCH KEYM_40][$Satisfies $ARCH KEYM_47]
 * \req [$Satisfies $SWS_Crypto 00148][$Satisfies $SWS_Crypto 91015][$Satisfies $SWS_Crypto 91004][$Satisfies $SWS_Crypto 91014]
 *
 * \brief Persist data in the HSM secure data storage
 *
 * \param[in] pHsmKey       Pointer to the HSM data slot in which the data has to be written
 * \param[in] pData         Pointer to a buffer containing the value to be written in the secure data store
 * \param[in] dataLength    Length of the data to be written in secure data store
 *
 * \retval E_OK                    Write operation successful
 * \retval CRYPTO_E_WRITE_FAIL     Write operation failed
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_WriteSecureDataStore(
    CONSTP2CONST(Crypto_11_Hsm_HsmKey, AUTOMATIC, CRYPTO_11_HSM_CONST) pHsmKey,
    CONSTP2CONST(uint8, AUTOMATIC, CRYPTO_11_HSM_CONST) pData,
    CONST(uint32, AUTOMATIC) dataLength);

/**
 * \req [$DD 66]
 * \req [$Satisfies $ARCH KEYM_14][$Satisfies $ARCH KEYM_8][$Satisfies $ARCH KEYM_37][$Satisfies $ARCH KEYM_40]
 * \req [$Satisfies $SWS_Crypto 00148][$Satisfies $SWS_Crypto 91015]
 *
 * \brief Performs an internal copy between HSM keys
 *
 * The source key is loaded (if not already loaded), then its key handle is used for a persist operation on the
 * target element HSM Flash key id. Then the source key is released from HSM RAM keystore.
 * In this way, the persisted keys will have the same content.
 * Finally source and destination keys are loaded if they are startup keys
 *
 * \param[in] pSourceHsmKey    Source Hsm key
 * \param[in] pTargetElement   Target key element
 * \return Error code indicating whether the internal copy operation was successful
 */
FUNC(Std_ReturnType, AUTOMATIC) Crypto_11_Hsm_CopyHsmKeyInternal(
        CONSTP2CONST(Crypto_11_Hsm_HsmKey, AUTOMATIC, CRYPTO_11_HSM_CONST) pSourceHsmKey,
        CONSTP2CONST(Crypto_11_Hsm_Config_KeyElement, AUTOMATIC, CRYPTO_11_HSM_CONST) pTargetElement);

/**
 * \req [$DD 78]
 * \req [$Satisfies $ARCH KEYM_55][$Satisfies $ARCH KEYM_54][$Satisfies $ARCH SHE_14][$Satisfies $ARCH SHE_18]
 *
 * \brief Exports SHE RAM key
 *
 * Depending on SHE services the function either uses the native or the SHE compatibility interface.
 * No checks are performed in this function if asserts are deactivated. Result buffer needs to have at least 144 bytes.
 *
 * \param[out] resultBuffer  pointer to struct of type ResultBuffer with at least 64 bytes to hold M1..M3 (112 bytes for M1..M5)
 * \return Error code indicating whether the SHE RAM key export was successfull
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ExportSheRamKey(
    CONSTP2VAR(ResultBuffer, AUTOMATIC, CRYPTO_11_HSM_VAR) resultBuffer);

#if (CRYPTO_11_HSM_CONFIG_ENABLE_EXTENSION == STD_ON)
/**
 * \req [$DD EXT_1]
 * \req [$Satisfies $ARCH VKMS_19] \req [$Satisfies $ARCH VKMS_21]
 *
 * \brief Resets the runtime data for a specific key in case the key was released using CSAI.
 *
 * This API resets the key state, saved job handle, (outdated) key handle, sets the key to invalid
 * and the number of sessions that use this key to zero.
 *
 * \note It is undefined behavior to call this function for a key that is actively used by any job or object.
**/
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ForgetHsmKey(
    CONSTP2CONST(Crypto_11_Hsm_HsmKey, AUTOMATIC, CRYPTO_11_HSM_CONST) pHsmKey);

/**
 * \req [$DD EXT_2]
 * \req [$Satisfies $ARCH VKMS_20]
 *
 * \brief Marks a key as loaded that was loaded or injected using CSAI.
 *
 * This API sets the key state to KEY_STATE_LOADED_NOT_IN_USE and sets the key handle to its key id.
 *
 * \note It is undefined behavior to call this function for a key that is actively used by any job or object.
 * \note The key must be a volatile key that is not persisted.
**/
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_AdoptHsmKey(
    CONSTP2CONST(Crypto_11_Hsm_HsmKey, AUTOMATIC, CRYPTO_11_HSM_CONST) pHsmKey);

/**
 * \req [$DD EXT_3]
 * \req [$Satisfies $ARCH VKMS_19]
 *
 * \brief Forgets multiple keys in the Crypto Driver based on a range of HSM key ids.
 *
 * This API calls Crypto_11_Hsm_ForgetHsmKey for each configured key in the range if HSM key ids.
 *
 * \note It is undefined behavior to call this function for a key that is actively used by any job or object.
**/
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ForgetHsmKeys_Range(
    CONST(uint32, AUTOMATIC) hsmKeyId,
    CONST(uint32, AUTOMATIC) numKeys
);

/**
 * \req [$DD EXT_4]
 * \req [$Satisfies $ARCH VKMS_20]
 *
 * \brief Marks multiple keys in the Crypto Driver as loaded based on a range of HSM key ids.
 *
 * This API calls Crypto_11_Hsm_AdoptHsmKey for each configured key in the range if HSM key ids.
 *
 * \note It is undefined behavior to call this function for a key that is actively used by any job or object.
 * \note All keys must be volatile keys that are not persisted.
**/
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_AdoptHsmKeys_Range(
    CONST(uint32, AUTOMATIC) hsmKeyId,
    CONST(uint32, AUTOMATIC) numKeys
);
#endif
#if (CRYPTO_11_HSM_USE_AUTOSAR_4_4_0_EXTENSIONS == STD_ON)

/**
 * \req [$DD 55]
 * \req [$Satisfies $SWS_Crypto 00203][$Satisfies $SWS_Crypto 00204]
 *
 * \brief Sets input and output pointers in case redirection is used.
**/
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_RedirectIO(
        CONSTP2VAR(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_VAR) job);

/**
 * \req [$DD 56]
 * \req [$Satisfies $SWS_Crypto 00204]
 *
 * \brief Data length of output key element is updated with the corresponding result length.
**/
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_RedirectionInvalidateOutputElements(
        CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_VAR) job,
        CONST(Std_ReturnType, AUTOMATIC) jobError);

#endif

#define CRYPTO_11_HSM_STOP_SEC_CODE
#include <Crypto_11_Hsm_MemMap.h>

#endif
