/**
 * \file
 * \copyright (c) ESCRYPT GmbH, Bochum, Germany 2019. All rights reserved, also regarding any disposal, exploitation, reproduction, editing, distribution, as well as in the event of applications for industrial property rights.
 */

/* Global QA-C suppression */
/* PRQA S 0776, 0778 EOF #Deviation ID directive_1.1_0776_0778 */
/* PRQA S 0292 EOF #Deviation ID directive_1.1_0292 */
/* PRQA S 0857 EOF #Deviation ID directive_1.1_0857 */
/* PRQA S 2053 EOF #Deviation ID rule_18.8_2053 */
/* PRQA S 3214 EOF #Suppressed because of the ASR memmap concept */
/* PRQA S 2000 EOF #Suppressed for simplicity. Not necessary having else statement is allowed. */
/* PRQA S 5002 EOF #Suppressed as header file names are defined by ASR. */

#ifndef CRYPTO_11_HSM_KEY_MANAGEMENT_H
#define CRYPTO_11_HSM_KEY_MANAGEMENT_H

#include <Csm_Types.h>
#include <Crypto_11_Hsm_Cfg.h>

#define CRYPTO_11_HSM_START_SEC_CODE
#include <Crypto_11_Hsm_MemMap.h>

/**
 * \req [$DD 28]
 * \req [$Satisfies $SWS_Crypto 91004][$Satisfies $SWS_Crypto 00116][$Satisfies $SWS_Crypto 00146][$Satisfies $SWS_Crypto 00189][$Satisfies $SWS_Crypto 00075][$Satisfies $SWS_Crypto 00076][$Satisfies $SWS_Crypto 00077][$Satisfies $SWS_Crypto 00078][$Satisfies $SWS_Crypto 00079]
 * \req [$Satisfies $ARCH KEYM_6][$Satisfies $ARCH KEYM_8][$Satisfies $ARCH KEYM_10][$Satisfies $ARCH KEYM_11][$Satisfies $ARCH KEYM_12][$Satisfies $ARCH KEYM_13][$Satisfies $ARCH KEYM_14][$Satisfies $ARCH KEYM_29][$Satisfies $ARCH KEYM_43][$Satisfies $ARCH KEYM_51][$Satisfies $ARCH KEYM_56][$Satisfies $ARCH KEYM_57]
 * \brief Sets the given key element bytes to the key identified by cryptoKeyId.
 *
 * \note When setting a new key element, the corresponding key given as parameter of this function is set as invalid.
 * If multiple keys are pointing this element, they will not be invalidated.
 *
 * \note This function works synchronously
 *
 * \param[in] cryptoKeyId Holds the identifier of the key whose key element shall be set.
 * \param[in] keyElementId Holds the identifier of the key element which shall be set.
 * \param[in] pKeyData Holds the pointer to the key data which shall be set as key element.
 * \param[in] keyLength Contains the length of the key element in bytes.
 *
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval CRYPTO_11_HSM_E_BUSY Request Failed, Crypto Driver Object is Busy
 * \retval CRYPTO_11_HSM_E_KEY_WRITE_FAIL Request failed because write access was denied
 * \retval CRYPTO_11_HSM_E_KEY_NOT_AVAILABLE Request failed because the key is not available.
 * \retval CRYPTO_11_HSM_E_KEY_SIZE_MISMATCH Request failed, key element size does not match size of provided data.
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyElementSet(
    VAR(uint32, AUTOMATIC) cryptoKeyId,
    VAR(uint32, AUTOMATIC) keyElementId,
    P2CONST(uint8, AUTOMATIC, CRYPTO_11_HSM_CONST) pKeyData,
    VAR(uint32, AUTOMATIC) keyLength
);
#define CRYPTO_11_HSM_SERVICE_ID_KEY_ELEMENT_SET 0x04u

/**
 * \req [$DD 29]
 * \req [$Satisfies $SWS_Crypto 00038][$Satisfies $SWS_Crypto 91005][$Satisfies $SWS_Crypto 00082][$Satisfies $SWS_Crypto 00083]
 * \brief Sets the key state of the key identified by cryptoKeyId to valid.
 * The interface is identical with Crypto_11_Hsm_KeySetValid but left for backward compatibility with older Autosar revisions.
 * \deprecated
 * \param[in] cryptoKeyId Holds the identifier of the key which shall be set to valid.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval CRYPTO_11_HSM_E_BUSY Request Failed, Crypro Driver Object is Busy
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyValidSet(
    VAR(uint32, AUTOMATIC) cryptoKeyId
);
#define CRYPTO_11_HSM_SERVICE_ID_KEY_VALID_SET 0x05u

/**
 * \req [$DD 30]
 * \req [$Satisfies $SWS_Crypto 00038][$Satisfies $SWS_Crypto 91014][$Satisfies $SWS_Crypto 00196][$Satisfies $SWS_Crypto 00197]
 * \brief Sets the key state of the key identified by cryptoKeyId to valid.
 * \param[in] cryptoKeyId Holds the identifier of the key which shall be set to valid.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval CRYPTO_11_HSM_E_BUSY Request Failed, Crypro Driver Object is Busy
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeySetValid(
    VAR(uint32, AUTOMATIC) cryptoKeyId
);
#define CRYPTO_11_HSM_SERVICE_ID_KEY_SET_VALID 0x05u

/**
 * \req [$DD 31]
 * \req [$Satisfies $SWS_Crypto 00092][$Satisfies $SWS_Crypto 00140][$Satisfies $SWS_Crypto 00139][$Satisfies $SWS_Crypto 91006][$Satisfies $SWS_Crypto 00085][$Satisfies $SWS_Crypto 00086][$Satisfies $SWS_Crypto 00087][$Satisfies $SWS_Crypto 00088][$Satisfies $SWS_Crypto 00089][$Satisfies $SWS_Crypto 00090]
 * \brief This interface shall be used to get a key element of the key identified by the cryptoKeyId and store the key element in the memory location pointed by the result pointer.
 * \note If the actual key element is directly mapped to flash memory, there could be a bigger delay when calling this function (synchronous operation).
 * \param[in] cryptoKeyId
 * \param[in] keyElementId
 * \param[out] resultPtr Holds the pointer of the buffer for the returned key element
 * \param[in,out] resultLengthPtr Holds a pointer to a memory location in which the length information is stored. On calling this function this parameter shall contain the size of the buffer provided by resultPtr. If the key element is configured to allow partial access, this parameter contains the amount of data which should be read from the key element. The size may not be equal to the size of the provided buffer anymore. When the request has finished, the amount of data that has been stored shall be stored.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval CRYPTO_11_HSM_E_BUSY Request Failed, Crypto Driver Object is Busy
 * \retval CRYPTO_11_HSM_E_KEY_NOT_AVAILABLE Request failed, the requested key element is not available
 * \retval CRYPTO_11_HSM_E_KEY_READ_FAIL Request failed because read access was denied
 * \retval CRYPTO_11_HSM_E_SMALL_BUFFER The provided buffer is too small to store the result
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyElementGet(
    VAR(uint32, AUTOMATIC) cryptoKeyId,
    VAR(uint32, AUTOMATIC) keyElementId,
    P2VAR(uint8, AUTOMATIC, CRYPTO_11_HSM_VAR) resultPtr,
    P2VAR(uint32, AUTOMATIC, CRYPTO_11_HSM_VAR) resultLengthPtr
);
#define CRYPTO_11_HSM_SERVICE_ID_KEY_ELEMENT_GET 0x06u

/**
 * \req [$DD 32]
 * \req [$Satisfies $SWS_Crypto 00146][$Satisfies $SWS_Crypto 00148][$Satisfies $SWS_Crypto 00154][$Satisfies $SWS_Crypto 00189][$Satisfies $SWS_Crypto 00149][$Satisfies $SWS_Crypto 00150][$Satisfies $SWS_Crypto 00151][$Satisfies $SWS_Crypto 00152][$Satisfies $SWS_Crypto 00153]
 * \req [$Satisfies $ARCH KEYM_37]
 * \brief Copies a key element to another key element in the same crypto driver.
 * \note If the actual key element is directly mapped to flash memory, there could be a bigger delay when calling this function (synchronous operation)
 * \param[in] cryptoKeyId Holds the identifier of the key whose key element shall be the source element.
 * \param[in] keyElementId Holds the identifier of the key element which shall be the source for the copy operation.
 * \param[in] targetCryptoKeyId Holds the identifier of the key whose key element shall be the destination element.
 * \param[in] targetKeyElementId Holds the identifier of the key element which shall be the destination for the copy operation.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval E_BUSY Request Failed, Crypto Driver Object is Busy
 * \retval CRYPTO_11_HSM_E_KEY_NOT_AVAILABLE Request failed, the requested key element is not available
 * \retval CRYPTO_11_HSM_E_KEY_READ_FAIL Request failed, not allowed to extract key element
 * \retval CRYPTO_11_HSM_E_KEY_WRITE_FAIL Request failed, not allowed to write key element.
 * \retval CRYPTO_11_HSM_E_KEY_SIZE_MISMATCH Request failed, key element sizes are not compatible.
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyElementCopy(
    VAR(uint32, AUTOMATIC) cryptoKeyId,
    VAR(uint32, AUTOMATIC) keyElementId,
    VAR(uint32, AUTOMATIC) targetCryptoKeyId,
    VAR(uint32, AUTOMATIC) targetKeyElementId
);
#define CRYPTO_11_HSM_SERVICE_ID_KEY_ELEMENT_COPY 0x0fu

#if (CRYPTO_11_HSM_USE_AUTOSAR_4_4_0_EXTENSIONS == STD_ON)
/**
 * \req [$DD 33]
 * \req [$Satisfies $SWS_Crypto 91015] [$Satisfies $SWS_Crypto 00207] [$Satisfies $SWS_Crypto 00208] [$Satisfies $SWS_Crypto 00209] [$Satisfies $SWS_Crypto 00210] [$Satisfies $SWS_Crypto 00211][$Satisfies $SWS_Crypto 00205][$Satisfies $SWS_Crypto 00206]
 * \req [$Satisfies $ARCH KEYM_40]
 * \brief Copies part of a key element from a given offset to another part of a key element to a given offset in the same crypto driver.
 * \note If the actual key element is directly mapped to flash memory, there could be a bigger delay when calling this function (synchronous operation).
 * \note This is an Autosar 4.4 functionality and has to be explicitly activated. Contrary to the Autosar 4.4 specification,
 * this function will not return the return code CRYPTO_E_KEY_EMPTY for uninitialized key elements since this return code does not exist
 * in Autosar 4.3.1. However, this should not pose a problem since key elements are automatically initialized during startup.
 * \param[in] cryptoKeyId Holds the identifier of the key whose key element shall be the source element.
 * \param[in] keyElementId Holds the identifier of the key element which shall be the source for the copy operation.
 * \param[in] keyElementSourceOffset Holds the offset in the key element which shall be the source for the copy operation.
 * \param[in] keyElementTargetOffset Holds the offset in the key element which shall be the destination for the copy operation.
 * \param[in] keyElementCopyLength Holds the length of the data which shall be copied.
 * \param[in] targetCryptoKeyId Holds the identifier of the key whose key element shall be the destination element.
 * \param[in] targetKeyElementId Holds the identifier of the key element which shall be the destination for the copy operation.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval E_BUSY Request Failed, Crypto Driver Object is Busy
 * \retval CRYPTO_11_HSM_E_KEY_NOT_AVAILABLE Request failed, the requested key element is not available
 * \retval CRYPTO_11_HSM_E_KEY_READ_FAIL Request failed, not allowed to extract key element
 * \retval CRYPTO_11_HSM_E_KEY_WRITE_FAIL Request failed, not allowed to write key element.
 * \retval CRYPTO_11_HSM_E_KEY_SIZE_MISMATCH Request failed, key element sizes are not compatible.
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyElementCopyPartial(
    VAR(uint32, AUTOMATIC) cryptoKeyId,
    VAR(uint32, AUTOMATIC) keyElementId,
    VAR(uint32, AUTOMATIC) keyElementSourceOffset,
    VAR(uint32, AUTOMATIC) keyElementTargetOffset,
    VAR(uint32, AUTOMATIC) keyElementCopyLength,
    VAR(uint32, AUTOMATIC) targetCryptoKeyId,
    VAR(uint32, AUTOMATIC) targetKeyElementId
);
#define CRYPTO_11_HSM_SERVICE_ID_KEY_ELEMENT_COPY_PARTIAL 0x13u
#endif

/**
 * \req [$DD 34]
 * \req [$Satisfies $SWS_Crypto 00155]
 * \brief Copies a key with all its elements to another key in the same crypto driver.
 * \param[in] cryptoKeyId Holds the identifier of the key whose key element shall be the source element.
 * \param[in] targetCryptoKeyId Holds the identifier of the key whose key element shall be the destination element.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval E_BUSY Request Failed, Crypto Driver Object is Busy
 * \retval CRYPTO_11_HSM_E_KEY_NOT_AVAILABLE Request failed, the requested key element is not available
 * \retval CRYPTO_11_HSM_E_KEY_READ_FAIL Request failed, not allowed to extract key element
 * \retval CRYPTO_11_HSM_E_KEY_WRITE_FAIL Request failed, not allowed to write key element.
 * \retval CRYPTO_11_HSM_E_KEY_SIZE_MISMATCH Request failed, key element sizes are not compatible.
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyCopy(
    VAR(uint32, AUTOMATIC) cryptoKeyId,
    VAR(uint32, AUTOMATIC) targetCryptoKeyId
);
#define CRYPTO_11_HSM_SERVICE_ID_KEY_COPY 0x10u

/**
 * \req [$DD 35]
 * \req [$Satisfies $SWS_Crypto 00160]
 * \brief Used to retrieve information which key elements are available in a given key.
 * \param[in] cryptoKeyId Holds the identifier of the key whose available element ids shall be exported.
 * \param[in] keyElementIdsPtr Holds a pointer to the memory location in which the number of key elements in the given key is stored. On calling this function, this parameter shall contain the size of the buffer provided by keyElementIdsPtr. When the request has finished, the actual number of key elements shall be stored.
 * \param[out] keyElementIdsLengthPtr Contains the pointer to the array where the ids of the key elements shall be stored.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval E_BUSY Request Failed, Crypto Driver Object is Busy
 * \retval CRYPTO_11_HSM_E_SMALL_BUFFER The provided buffer is too small to store the result
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyElementIdsGet(
    VAR(uint32, AUTOMATIC) cryptoKeyId,
    P2VAR(uint32, AUTOMATIC, CRYPTO_11_HSM_VAR) keyElementIdsPtr,
    P2VAR(uint32, AUTOMATIC, CRYPTO_11_HSM_VAR) keyElementIdsLengthPtr
);
#define CRYPTO_11_HSM_SERVICE_ID_KEY_ELEMENT_IDS_GET 0x11u

/**
 * \req [$DD 36]
 * \req [$Satisfies $SWS_Crypto 91013][$Satisfies $SWS_Crypto 00128][$Satisfies $SWS_Crypto 00129][$Satisfies $SWS_Crypto 00130][$Satisfies $SWS_Crypto 00131]
 * \brief This function generates the internal seed state using the provided entropy source. Furthermore, this function can be used to update the seed state with new entropy
 * \param[in] cryptoKeyId Holds the identifier of the key for which a new seed shall be generated.
 * \param[in] seedPtr Holds a pointer to the memory location which contains the data to feed the seed.
 * \param[in] seedLength Contains the length of the seed in bytes.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_RandomSeed(
    VAR(uint32, AUTOMATIC) cryptoKeyId,
    P2CONST(uint8, AUTOMATIC, CRYPTO_11_HSM_CONST) seedPtr,
    VAR(uint32, AUTOMATIC) seedLength
);
#define CRYPTO_11_HSM_SERVICE_ID_RANDOM_SEED 0x0du

/**
 * \req [$DD 37]
 * \req [$Satisfies $ARCH KEYM_46] [$Satisfies $SWS_Crypto 91007] [$Satisfies $SWS_Crypto 00165] [$Satisfies $SWS_Crypto 00094] [$Satisfies $SWS_Crypto 00095]
 * \brief Generates new key material store it in the key identified by cryptoKeyId.
 *
 * Note that asymmetric keys can only be generated in HSM key pair slots.
 *
 * \param[in] cryptoKeyId Holds the identifier of the key which is to be updated with the generated value.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval E_BUSY Request Failed, Crypto Driver Object is Busy
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyGenerate(
    VAR(uint32, AUTOMATIC) cryptoKeyId
);
#define CRYPTO_11_HSM_SERVICE_ID_KEY_GENERATE 0x07u

/**
 * \req [$DD 38]
 * \req [$Satisfies $SWS_Crypto 91008]
 * \brief Derives a new key by using the key elements in the given key identified by the cryptoKeyId. The given key contains the key elements for the password, salt. The derived key is stored in the key element with the id 1 of the key identified by targetCryptoKeyId. The number of iterations is given in the key element CRYPTO_11_HSM_KE_KEYDERIVATION_ITERATIONS.
 * \param[in] cryptoKeyId Holds the identifier of the key which is used for key derivation.
 * \param[in] targetCryptoKeyId Holds the identifier of the key which is used to store the derived key.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval E_BUSY Request Failed, Crypto Driver Object is Busy
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyDerive(
    VAR(uint32, AUTOMATIC) cryptoKeyId,
    VAR(uint32, AUTOMATIC) targetCryptoKeyId
);
#define CRYPTO_11_HSM_SERVICE_ID_KEY_DERIVE 0x08u

/**
 * \req [$DD 39]
 * \req [$Satisfies $SWS_Crypto 91009] [$Satisfies $SWS_Crypto 00103] [$Satisfies $SWS_Crypto 00104] [$Satisfies $SWS_Crypto 00105]
 * \req [$Satisfies $SWS_Crypto 00106] [$Satisfies $SWS_Crypto 00107] [$Satisfies $SWS_Crypto 00109] [$Satisfies $SWS_Crypto 00110] [$Satisfies $SWS_Crypto 00167]
 * \brief Calculates the public value for the key exchange and stores the public key in the memory location pointed by the public value pointer.
 * \param[in] cryptoKeyId Holds the identifier of the key which shall be used for the key exchange protocol.
 * \param[out] publicValuePtr Contains the pointer to the data where the public value shall be stored.
 * \param[in,out] publicValueLengthPtr Holds a pointer to the memory location in which the public value length information is stored. On calling this function, this parameter shall contain the size of the buffer provided by publicValuePtr. When the request has finished, the actual length of the returned value shall be stored.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval E_BUSY Request Failed, Crypto Driver Object is Busy
 * \retval CRYPTO_11_HSM_E_SMALL_BUFFER The provided buffer is too small to store the result
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyExchangeCalcPubVal(
    VAR(uint32, AUTOMATIC) cryptoKeyId,
    P2VAR(uint8, AUTOMATIC, CRYPTO_11_HSM_VAR) publicValuePtr,
    P2VAR(uint32, AUTOMATIC, CRYPTO_11_HSM_VAR) publicValueLengthPtr
);
#define CRYPTO_11_HSM_SERVICE_ID_KEY_EXCHANGE_CALC_PUB_VAL 0x09u

/**
 * \req [$DD 40]
 * \req [$Satisfies $SWS_Crypto 91010] [$Satisfies $SWS_Crypto 00111] [$Satisfies $SWS_Crypto 00112]
 * \req [$Satisfies $SWS_Crypto 00113] [$Satisfies $SWS_Crypto 00115]
 * \brief Calculates the shared secret key for the key exchange with the key material of the key identified by the cryptoKeyId and the partner public key. The shared secret key is stored as a key element in the same key.
 * \param[in] cryptoKeyId Holds the identifier of the key which shall be used for the key exchange protocol.
 * \param[in] partnerPublicValuePtr Holds the pointer to the memory location which contains the partner's public value.
 * \param[in] partnerPublicValueLength Contains the length of the partner's public value in bytes.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval E_BUSY Request Failed, Crypto Driver Object is Busy
 * \retval CRYPTO_11_HSM_E_SMALL_BUFFER The provided buffer is too small to store the result
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyExchangeCalcSecret(
    CONST(uint32, AUTOMATIC) cryptoKeyId,
    P2CONST(uint8, AUTOMATIC, CRYPTO_11_HSM_CONST) partnerPublicValuePtr,
    VAR(uint32, AUTOMATIC) partnerPublicValueLength
);
#define CRYPTO_11_HSM_SERVICE_ID_KEY_EXCHANGE_CALC_SECRET 0x0au

/**
 * \req [$DD 41]
 * \req [$Satisfies $SWS_Crypto 91011]
 * \brief Parses the certificate data stored in the key element CRYPTO_11_HSM_KE_CERT_DATA and fills the key elements CRYPTO_11_HSM_KE_CERT_SIGNEDDATA, CRYPTO_11_HSM_KE_CERT_PARSEDPUBLICKEY and CRYPTO_11_HSM_KE_CERT_SIGNATURE.
 * \param[in] cryptoKeyId Holds the identifier of the key which shall be parsed.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval E_BUSY Request Failed, Crypto Driver Object is Busy
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_CertificateParse(
    VAR(uint32, AUTOMATIC) cryptoKeyId
);
#define CRYPTO_11_HSM_SERVICE_ID_CERTIFICATE_PARSE 0x0bu

/**
 * \req [$DD 42]
 * \req [$Satisfies $SWS_Crypto 00171]
 * \brief Verifies the certificate stored in the key referenced by cryptoValidateKeyId with the certificate stored in the key referenced by cryptoKeyId.
 * \param[in] cryptoKeyId Holds the identifier of the key which shall be used to validate the certificate.
 * \param[in] verifyCryptoKeyId Holds the identifier of the key contain
 * \param[out] verifyPtr Holds a pointer to the memory location which will contain the result of the certificate verification.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request Failed
 * \retval E_BUSY Request Failed, Crypto Driver Object is Busy
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_CertificateVerify(
    VAR(uint32, AUTOMATIC) cryptoKeyId,
    VAR(uint32, AUTOMATIC) verifyCryptoKeyId,
    P2VAR(Crypto_VerifyResultType, AUTOMATIC, CRYPTO_11_HSM_VAR) verifyPtr
);
#define CRYPTO_11_HSM_SERVICE_ID_CERTIFICATE_VERIFY 0x12u

#if ((CRYPTO_11_HSM_CONFIG_HAS_ALGOFAM_ECCNIST == STD_ON) || (CRYPTO_11_HSM_CONFIG_HAS_ALGOFAM_ED25519 == STD_ON))
/**
 * \req [$DD 48]
 * \req [$Satisfies $ARCH KEYM_35]
 * \brief Initializes variables used in the Key Management module */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyManagement_Initialize(void);
#endif

/**
 * \req [$DD 77]
 * \req [$Satisfies $ARCH ENC_2] [$Satisfies $ARCH DEC_2]
 * \brief Reads the data of a key element for Crypto Driver internal use (e.g. IV)
 * \param[in] cryptoKeyId
 * \param[in] keyElementId
 * \param[in] length Length of the data to read
 * \param[out] resultPtr Holds the pointer of the buffer for the returned key element
 * \retval E_OK Requested amount of data read
 * \retval CRYPTO_E_KEY_NOT_AVAILABLE The key element does not exist
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_KeyManagement_InternalGet(
    CONST(uint32, AUTOMATIC) cryptoKeyId,
    CONST(uint32, AUTOMATIC) keyElementId,
    CONST(uint32, AUTOMATIC) length,
    CONSTP2VAR(uint8, AUTOMATIC, CRYPTO_11_HSM_CONST) resultPtr
);

#define CRYPTO_11_HSM_STOP_SEC_CODE
#include <Crypto_11_Hsm_MemMap.h>

#endif
